% IntervalsCell.m -- for fixed g12 and g34, find the intervals based on the number of positive steady state solutions
% Copyright 2016
% Jonathan Hauenstein

function [LeftPt,RightPt,NumSolns] = IntervalsCell(g12,g34)
% compute the intervals for 0 <= g23 <= 0.1 regarding the number of positive steady state solutions
LeftPt = 0; RightPt = 0; NumSolns = 0;

% compute the possible endpoints by computing Discriminant
cd Disc
CreateStart([g12;g34;0;0;0;0],6,1,'final_parameters');
eval(['!mpirun -np 32 bertini input_disc_param start_171 > /dev/null']);
[nD,D] = ReadInStart(9,'finite_solutions');
Soln = D(:,1:4);
G23 = D(:,5);

% find the ones in [0,0.1] corresponding to real solutions 
Good = zeros(1,nD);
for j = 1:nD
  if abs(imag(G23(j))) < 1e-8 
    % G23(j) is real
    G23(j) = real(G23(j));
    if 0 < G23(j) && G23(j) < 0.1 && norm(imag(Soln(j,:))) < 1e-8
      % Good one
      Good(j) = 1;
    end;
  end;
end;

% sort the discriminant values -- find only the distinct ones
G23 = sort(G23(Good == 1),'ascend');
d23 = [1; diff(G23)];
G23 = G23(d23 > 1e-8);

% setup initial intervals
LeftPt = [0 G23'];
RightPt = [G23' 0.1];
MidPt = (RightPt + LeftPt)/2;
numIntervals = length(LeftPt);
NumSolns = zeros(1,numIntervals);
disp(['num intervals: ',num2str(numIntervals)]);

% for each interval, compute number of positive steady state solutions 
cd ../Solve
for j = 1:numIntervals
  % solve at the midpoint
  CreateStart([g12;MidPt(j);g34],3,1,'final_parameters');
  eval(['!mpirun -np 1 bertini input_param start_81 > /dev/null']);
  [nP,P] = ReadInStart(4,'finite_solutions');
  if nP ~= 81
    disp(['Error in computing the solutions!']);
    return;
  end;
  Good = zeros(1,nP);
  for k = 1:nP
    if norm(imag(P(k,:))) < 1e-8
      % P(k,:) is real
      P(k,:) = real(P(k,:));
      if min(P(k,:)) > 0
        % P(k,:) is positive
        Good(k) = 1;
      end;
    end;
  end;
  P = P(Good == 1,:);
  for k = 1:size(P,1)
    M = evalJac(P(k,:),g12,MidPt(j),g34);
    if max(eig(M)) < 1e-8 
      % stable solution
      NumSolns(j) = NumSolns(j) + 1;
    end;
  end;
  disp(['(',num2str(LeftPt(j)),',',num2str(RightPt(j)),'): ',num2str(NumSolns(j))]);
end;
cd ../

% merge intervals and return answer
[LeftPt,RightPt,NumSolns] = MergeIntervals(LeftPt,RightPt,NumSolns);

return;



function [L,R,N] = MergeIntervals(Lin,Rin,Nin)

numOrig = length(Lin);
numNew = 0;
Good = ones(1,numOrig);
for j = 1:numOrig
  if Good(j) == 1
    % start of a new interval
    numNew = numNew + 1;
    L(numNew) = Lin(j);
    R(numNew) = Rin(j);
    N(numNew) = Nin(j);
  
    % look to merge intervals
    for k = j+1:numOrig
      if Nin(j) == Nin(k)
        % merge these intervals
        R(numNew) = Rin(k);
        Good(k) = 0;
      else
        % stop looking to merge intervals
        break;
      end;
    end;
  end;
end;

return;



function [J] = evalJac(x,g12,g23,g34)

a = 0.4;
eps = 0.01;

J = zeros(4,4);

J(1,1) = -3*x(1)^2+2*(1+a+eps)*x(1)-a*eps-a-eps-g12;
J(1,2) = g12;
J(2,1) = g12;
J(2,2) = -3*x(2)^2+2*x(2)*(1+a+eps)-a*eps-a-eps-g23-g12;
J(2,3) = g23;
J(3,2) = g23;
J(3,3) = -3*x(3)^2+2*x(3)*(1+a+eps)-a*eps-a-eps-g34-g23;
J(3,4) = g34;
J(4,3) = g34;
J(4,4) = -3*x(4)^2+2*x(4)*(1+a+eps)-a*eps-a-eps-g34;

return;


